@@ -23,7 +23,7 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <stdio.h>
-#include <stdint.h>
+#include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
